package cc.fascinated.bat.common;

import cc.fascinated.bat.service.DiscordService;
import lombok.experimental.UtilityClass;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;

/**
 * @author Fascinated (fascinated7)
 */
@UtilityClass
@Log4j2(topic = "ChannelUtils")
public class ChannelUtils {
    /**
     * Gets the user with the given id
     *
     * @param id      the id of the user
     * @param retries the amount of retries
     * @return the user with the given id
     */
    private static TextChannel getTextChannel(String id, int retries) {
        if (retries >= 25) {
            log.error("Failed to find user \"{}\" after {} retries.", id, retries);
            return null;
        }
        if (id == null) {
            return null;
        }
        TextChannel channel = DiscordService.JDA.getTextChannelById(id);
        if (channel == null) {
            return getTextChannel(id, retries + 1);
        }
        if (retries >= 5) {
            log.info("Found text channel \"{}\" after {} retries.", channel.getName(), retries);
        }
        return channel;
    }

    /**
     * Gets the user with the given id
     *
     * @param id the id of the user
     * @return the user with the given id
     */
    public static TextChannel getTextChannel(String id) {
        return getTextChannel(id, 0);
    }
}
