package cc.fascinated.bat.features.base.commands.server.channel;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.Channel;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Nick (okNick)
 */
@Component
@CommandInfo(name = "viewtopic", description = "View the topic of a channel")
public class ViewTopicSubCommand extends BatCommand {
    public ViewTopicSubCommand() {
        super.addOptions(new OptionData(OptionType.CHANNEL, "channel", "The channel to view the topic of", false));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        Channel target = event.getOption("channel") == null ? channel : event.getOption("channel").getAsChannel();
        if (!(target instanceof TextChannel textChannel)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("<#%s> is not a text channel!".formatted(target.getId()))
                            .build())
                    .queue();
            return;
        }

        String topic = textChannel.getTopic();
        if (topic == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("<#%s> does not have a topic!".formatted(textChannel.getId()))
                            .build())
                    .queue();
            return;
        }

        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setDescription("The topic of <#%s> is: \"%s\"".formatted(textChannel.getId(), topic))
                .build()
        ).queue();
    }
}