package cc.fascinated.bat.features.counter;

import cc.fascinated.bat.common.ChannelUtils;
import cc.fascinated.bat.common.NumberUtils;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
@Setter
public class CounterChannel {
    /**
     * The channel id of this counter channel
     */
    private final String channelId;

    /**
     * The current count of this counter channel
     */
    private int currentCount;

    /**
     * Whether the counter should reset when a user
     * inputs the wrong number or a non number
     */
    private boolean breakable;

    /**
     * Increment the counter
     *
     * @param input the input to increment the counter with
     * @return the return type of the increment
     */
    public CounterResult incrementChannel(String input) {
        int number = NumberUtils.parseInt(input);

        // The number is invalid
        if (number == -1 || number != this.nextNumber()) {
            // The counter was broken
            if (this.isBreakable()) {
                this.reset();
                return CounterResult.BROKEN;
            }
            return CounterResult.INVALID_NUMBER;
        }

        this.currentCount++;
        return CounterResult.SUCCESS;
    }

    /**
     * Gets the next number for this counter
     */
    public int nextNumber() {
        return currentCount + 1;
    }

    /**
     * Resets the current count
     * <p>
     * This usually happens if the counter is broken
     * </p>
     */
    public void reset() {
        currentCount = 0;
    }

    /**
     * Gets the channel
     *
     * @return the channel
     */
    public TextChannel getChannel() {
        return ChannelUtils.getTextChannel(channelId);
    }
}
