package cc.fascinated.bat.features.counter;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.FeatureProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class CountingListener implements EventListener {
    private final CounterFeature counterFeature;

    @Autowired
    public CountingListener(@NonNull CounterFeature counterFeature) {
        this.counterFeature = counterFeature;
    }

    @Override
    public void onGuildMessageReceive(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull MessageReceivedEvent event) {
        FeatureProfile featureProfile = guild.getFeatureProfile();
        if (featureProfile.isFeatureDisabled(counterFeature)) {
            return;
        }
        CounterProfile counterProfile = guild.getCounterProfile();
        if (counterProfile == null) {
            return;
        }
        CounterChannel counterChannel = counterProfile.getCounterChannel(event.getChannel().getId());
        if (counterChannel == null) {
            return;
        }
        CounterResult result = counterChannel.incrementChannel(event.getMessage().getContentRaw());
        if (result == null) {
            return;
        }

        switch (result) {
            case SUCCESS -> event.getMessage().addReaction(Emoji.fromUnicode("✅")).queue();
            case INVALID_NUMBER -> event.getMessage().addReaction(Emoji.fromUnicode("❌")).queue();
            case BROKEN -> {
                event.getMessage().addReaction(Emoji.fromUnicode("❌")).queue();
                event.getMessage().reply("%s, you have broken the counter!".formatted(event.getAuthor().getAsMention())).queue();
            }
        }
    }
}
