package cc.fascinated.bat.features.logging;

import cc.fascinated.bat.common.ChannelUtils;
import cc.fascinated.bat.common.Serializable;
import com.google.gson.Gson;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.bson.Document;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
public class LogProfile extends Serializable {
    /**
     * The log channels for this profile
     */
    private final Map<LogType, String> logChannels = new HashMap<>();

    /**
     * Checks if the log channel for the specified log type exists
     *
     * @param logType - the log type
     * @return true if it exists, false otherwise
     */
    public boolean hasLogChannel(LogType logType) {
        return this.logChannels.containsKey(logType);
    }

    /**
     * Gets the log channel for the specified log type
     *
     * @param logType - the log type
     * @return the log channel, or null if it doesn't exist
     */
    public TextChannel getLogChannel(LogType logType) {
        return ChannelUtils.getTextChannel(this.logChannels.get(logType));
    }

    /**
     * Sets the log channel for the specified log type
     *
     * @param logType - the log type
     * @param channel - the channel
     */
    public void setLogChannel(LogType logType, TextChannel channel) {
        this.logChannels.put(logType, channel.getId());
    }

    /**
     * Removes the log channel for the specified log type
     *
     * @param logType - the log type
     */
    public void removeLogChannel(LogType logType) {
        this.logChannels.remove(logType);
    }

    @Override
    public void load(Document document, Gson gson) {
        for (LogType logType : LogType.values()) {
            if (document.containsKey(logType.name())) {
                TextChannel channel = ChannelUtils.getTextChannel(document.getString(logType.name()));
                if (channel == null) {
                    return;
                }
                this.logChannels.put(logType, channel.getId());
            }
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        for (Map.Entry<LogType, String> entry : this.logChannels.entrySet()) {
            document.append(entry.getKey().name(), entry.getValue());
        }
        return document;
    }

    @Override
    public void reset() {
        this.logChannels.clear();
    }
}
