package cc.fascinated.bat.features.scoresaber.command.userfeed;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.scoresaber.profile.guild.UserScoreFeedProfile;
import cc.fascinated.bat.features.scoresaber.profile.user.ScoreSaberProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber-user-feed:user.sub")
@CommandInfo(name = "user", description = "Adds or removes a user from the feed")
public class UserSubCommand extends BatCommand {
    private final GuildService guildService;
    private final UserService userService;

    @Autowired
    public UserSubCommand(GuildService guildService, UserService userService) {
        super.addOptions(new OptionData(OptionType.USER, "user", "Add or remove a user from the score feed", false));
        this.guildService = guildService;
        this.userService = userService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        UserScoreFeedProfile profile = guild.getProfile(UserScoreFeedProfile.class);
        OptionMapping option = event.getOption("user");
        if (option == null) {
            if (profile.getTrackedUsers().isEmpty()) {
                event.replyEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("There are no users being tracked in the feed")
                        .build()).queue();
                return;
            }
            StringBuilder stringBuilder = new StringBuilder();
            for (String accountId : profile.getTrackedUsers()) {
                stringBuilder.append("[%s](%s)".formatted(
                        accountId,
                        "https://scoresaber.com/u/%s".formatted(accountId)
                ));
            }
            event.replyEmbeds(EmbedUtils.genericEmbed()
                    .setDescription("The current users being tracked in the feed are:\n%s".formatted(stringBuilder.toString()))
                    .build()).queue();
            return;
        }

        User target = option.getAsUser();
        BatUser targetUser = userService.getUser(target.getId());
        ScoreSaberProfile targetProfile = targetUser.getScoreSaberProfile();
        if (targetProfile.getAccountId() == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The user you are trying to track does not have a linked ScoreSaber profile")
                    .build()).queue();
            return;
        }

        boolean added = false;
        if (profile.isUserTracked(targetProfile.getAccountId())) {
            profile.removeTrackedUser(targetProfile.getAccountId());
        } else {
            profile.addTrackedUser(targetProfile.getAccountId());
            added = true;
        }

        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully %s %s from the feed".formatted(
                        added ? "added" : "removed",
                        target.getAsMention()
                ))
                .build()).queue();
    }
}
